@@ -13,6 +13,7 @@ import pandas as pd
 import scipy
 import scanpy as sc
 import copy
+from sklearn.mixture import GaussianMixture
 
 #from auxi import rm_outliers
 def rm_outliers(x):
@@ -145,9 +146,10 @@ def ratios_bc(adata_mg, dblgroup, max_iter=100, tol=1e-3, n_logR=100, n_p=1000,
         if sum(replace) > 0:
 
             w_best[replace] = w_best_tmp[replace]
-            #log2R_m,log2R_std = norm.fit(rm_outliers(np.log2((1-w_best)/w_best)))
-            log2R_m,log2R_std = norm.fit(np.log2((1-w_best)/w_best))
-            #log2R_std = np.max([log2R_std,log2R_std_prior])
+            log2R_m,log2R_std = norm.fit(rm_outliers(np.log2((1-w_best)/w_best)))
+            #log2R_m,log2R_std = norm.fit(np.log2((1-w_best)/w_best))
+            if log2R_std < 0.01: #
+                log2R_std = np.max([log2R_std,log2R_std_prior])
 
             ll_best[2,replace] = logR_ll_max[replace]
             ll_best[3,replace] = mult_ll_max[replace]
@@ -215,6 +217,18 @@ def ratios_bc(adata_mg, dblgroup, max_iter=100, tol=1e-3, n_logR=100, n_p=1000,
     adata_mg.uns['ratio']['ll_best'] = ll_best
     adata_mg.uns['ratio']['niter'] += niter
 
+    ## check confidence
+    r_list = (1-w_best)/w_best
+    if len(r_list) < 15:
+        log2r = np.log2(r_list)
+    else:
+        log2r = rm_outliers(np.log2(r_list))
+
+    unimodal = GaussianMixture(1).fit(log2r.reshape(-1,1))
+    bimodal =  GaussianMixture(2).fit(log2r.reshape(-1,1))
+    lrt = -2*(unimodal.score_samples(log2r.reshape(-1,1)).sum() - bimodal.score_samples(log2r.reshape(-1,1)).sum())
+    pval = scipy.stats.chi2.sf(lrt, 3)
+    adata_mg.uns['ratio']['confidence'] = pval
 
 
 
@@ -331,28 +345,14 @@ def get_dbl_mg(adata,groupby,groups='all',output=None, num_mg = 100, kl_cutoff=N
 
     return adata_mgdic
     
-    adata_mgdic = {}
-    for dblgroup in groups:
-        mg_tmp = get_dbl_mg_bc(adata, 
-                               groupby, 
-                               dblgroup, 
-                               output=output,
-                               num_mg=num_mg, 
-                               kl_cutoff=kl_cutoff,
-                               merging_threshold=merging_threshold,
-                               skip_threshold=skip_threshold,
-                               alphaMin=alphaMin)
-        adata_mgdic[dblgroup] = mg_tmp
 
-    return adata_mgdic
-    
+
 
 
 
 def get_dbl_mg_bc(adata, groupby, dblgroup, output=None, num_mg = 100, kl_cutoff=None, merging_threshold=5, skip_threshold=2,alphaMin = 1):
     '''
     Extract metagenes from raw UMI counts of a certain type of heterotypic doublets.
-
     Parameters
     ----------
     adata : AnnData
@@ -375,70 +375,78 @@ def get_dbl_mg_bc(adata, groupby, dblgroup, output=None, num_mg = 100, kl_cutoff
          this gene is skipped. The default is 2.
     alphaMin : float, optional
          Individual genes with alpha greater than the threshold are skipped in mergging step and considerted as a specifial metagene with one member gene. The default is 1.
-
     Returns
     -------
     adata_dbl_mg_top : AnnData
         The UMI count matrix of shape `n_obs` × `n_metangenes`.
         Rows correspond to droplets and columns to metagenes.
-
     '''
-    groups = dblgroup.split('_')+[dblgroup]
-    raw_Alpha1_0 = adata.varm['para_diri'][groups[0]]
-    raw_Alpha2_0 = adata.varm['para_diri'][groups[1]]
-    
+    groups = dblgroup.split('_') #+[dblgroup]
+
+    adata_dbl = adata[adata.obs[groupby]==dbl,:].copy()
+    adata_dbl = adata_dbl[:,adata_dbl.X.sum(0)>0]
+
+
+    count_dbl = adata_dbl[adata_dbl.obs[groupby]==dblgroup,:].X
+    if isinstance(count_dbl, scipy.sparse.csr_matrix):
+        count_dbl = count_dbl.toarray()
+
+
+    raw_Alpha1_0 = adata_dbl.varm['para_diri'][groups[0]]
+    raw_Alpha2_0 = adata_dbl.varm['para_diri'][groups[1]]
+
+
     if isinstance(kl_cutoff,float):
         # use genes with KL greater than cutoff to obtain meta-genes
-        kl = adata.varm['kl']
+        kl = adata_dbl.varm['kl']
         KL_filter_idx = np.where(kl > kl_cutoff)[0]
     else:
         KL_filter_idx = np.array([i for i in range(len(raw_Alpha1_0))])
-        
+
     raw_Alpha1 = raw_Alpha1_0[KL_filter_idx]
     raw_Alpha2 = raw_Alpha2_0[KL_filter_idx]
 
     mg_dic = get_mg(raw_Alpha1, raw_Alpha2, merging_threshold, skip_threshold,alphaMin)
-    
+
     mg_pool = mg_dic['mg_pool'] 
 
     mg_gidx = [list(KL_filter_idx[term]) for term in mg_pool]
     mg_genepool = [g for sub in  mg_gidx  for g in sub]
-    
+
     mg_alpha1 = mg_dic['alpha1']
     mg_alpha2 = mg_dic['alpha2']
-    
+
     left_gidx = list(set(range(len(raw_Alpha1_0))).difference(mg_genepool)) 
     mg_gidx.append(left_gidx)
-    
+
     # merge alpha and counts according to meta-genes
-    count_dbl = adata[adata.obs[groupby]==groups[2],:].X
-    if isinstance(count_dbl, scipy.sparse.csr.csr_matrix):
-        count_dbl = count_dbl.toarray()
-        
+
     mg_Alpha_arr = np.concatenate((np.array([mg_alpha1, mg_alpha2]), np.array([[sum(raw_Alpha1_0)-sum(mg_alpha1)],[sum(raw_Alpha2_0)-sum(mg_alpha2)]])),axis=1)
     Y = np.transpose(np.array([np.sum(count_dbl[:,term],1) for term in mg_gidx]))
-    
-    adata_dbl_mg = sc.AnnData(Y)
-    adata_dbl_mg.obs_names = adata.obs_names[adata.obs[groupby]==groups[2]]
+
+    adata_dbl_mg = sc.AnnData(Y,dtype=Y.dtype)
+    adata_dbl_mg.obs_names = adata_dbl.obs_names#[adata_dbl.obs[groupby]==dblgroup]
     adata_dbl_mg.varm['para_diri'] = mg_Alpha_arr.transpose()
     if output is not None:
         if not os.path.exists(output):
             os.makedirs(output)
-        adata_dbl_mg.write_h5ad(os.path.join(output,groups[2]+'_dbl_mg.h5ad'))
+        adata_dbl_mg.write_h5ad(os.path.join(output,dblgroup+'_dbl_mg.h5ad'))
 
     # extract top num_mg 
     topmg_idx = [[i] for i in range(num_mg)] + [list(range(num_mg,  len(mg_gidx)))]
     mg_Alpha_new = np.transpose(np.array([np.sum(mg_Alpha_arr[:,term],1) for term in topmg_idx]))
     Y_new = np.transpose(np.array([np.sum(Y[:,term],1) for term in topmg_idx]))
-    
-    adata_dbl_mg_top = sc.AnnData(Y_new)
-    adata_dbl_mg_top.obs_names = adata.obs_names[adata.obs[groupby]==groups[2]]
+
+    adata_dbl_mg_top = sc.AnnData(Y_new, dtype=Y_new.dtype)
+    adata_dbl_mg_top.obs_names = adata_dbl.obs_names#[adata.obs[groupby]==dblgroup]
     adata_dbl_mg_top.varm['para_diri'] = mg_Alpha_new.transpose()
     if len(adata.uns.keys()):
         for val in adata.uns.keys():
             adata_dbl_mg_top.uns[val] = adata.uns[val]
 
-    return adata_dbl_mg_top        
+    return adata_dbl_mg_top
+
+
 
 
 
@@ -812,8 +820,8 @@ def heteroDbl_bc(adata, dbl, d_groupby, ct_groupby, g2meta = None, de_sorted=Non
     if vis:
         
         dbl_sub['pred'] = ['others']*dbl_sub.shape[0]
-        dbl_sub.loc[dbl_onhit,'pred'] = '-'.join(ct_pair)
-        dbl_sub['pred'] = pd.Categorical(dbl_sub['pred'],categories=['-'.join(ct_pair), 'others'])
+        dbl_sub.loc[dbl_onhit,'pred'] = '_'.join(ct_pair)
+        dbl_sub['pred'] = pd.Categorical(dbl_sub['pred'],categories=['_'.join(ct_pair), 'others'])
 
 
         fig = plt.figure(figsize=(11,4),dpi=64)
