@@ -120,25 +120,31 @@ def alpha_opt_process(record, saveFig = False, filename=None):
 
     plt.show()
 '''
+import math
 
-
-def logRatio_dist(r_list,nbins=20,return_fig=None,rm_outlier=True):
+def logRatio_dist(r_list,nbins=20,return_fig=None,rm_outlier=True,fig_size=(4,4),dpi=64):
     
     #output = para.get('output',None)
     custom_palette = sns.color_palette("Greens",5)
     
-    xy_lim = [np.log2(1)-6, np.log2(1)+6] #[np.floor(v_min), np.floor(v_max)]
-    xy_ticks = np.arange(xy_lim[0]+2, xy_lim[1], step=2)
-
     x = np.log2(r_list)
+    
+    xy_lim = [math.floor(np.min(x)), math.ceil(np.max(x))] #[np.log2(1)-6, np.log2(1)+6] #[np.floor(v_min), np.floor(v_max)]
+    x_m = round(np.mean(x))
+    x_margin = np.max(np.array(xy_lim)-x_m)
+    #xy_ticks = np.arange(xy_lim[0]+2, xy_lim[1], step=2)
+    xy_ticks = np.arange(x_m-x_margin, x_m+x_margin+1, step=1)
+    xy_ticks = xy_ticks[xy_ticks>0]
+    
     if rm_outlier:
         x_shrinkage = rm_outliers(x)
     else:
         x_shrinkage = x
         
     #plt.figure(figsize=(6,8),dpi=256)
-    fig, (ax_box,ax_hist) = plt.subplots(2, sharex=True, gridspec_kw={"height_ratios": (.1, .9)})
-
+    fig, (ax_box,ax_hist) = plt.subplots(2, sharex=True, dpi=dpi,gridspec_kw={"height_ratios": (.1, .9)})
+    fig.set_figheight(fig_size[0])
+    fig.set_figwidth(fig_size[1])
     sns.boxplot(data=x, ax=ax_box, orient="h",fliersize=1,color = custom_palette[4])
     ax_box.set(xlabel='')
 
@@ -146,8 +152,8 @@ def logRatio_dist(r_list,nbins=20,return_fig=None,rm_outlier=True):
     yy_shrinkage = scipy.stats.norm.pdf(xx,np.mean(x_shrinkage),np.std(x_shrinkage))
 
     a = plt.hist(x,nbins,label='trueR',alpha=0.5, color=custom_palette[4],density=True)
-    top = np.ceil( np.max(a[0])/5 )*5
-    plt.plot([np.mean(x_shrinkage), np.mean(x_shrinkage)], [0,0.3], linestyle='dashed', color='black')
+    top = np.max(a[0])*1.05
+    plt.plot([np.mean(x_shrinkage), np.mean(x_shrinkage)], [0,top], linestyle='dashed', color='black')
     plt.plot(xx,yy_shrinkage,label='rm outliers',color='black')
     plt.xlabel('Total mRNA ratio',fontsize=18)
     plt.ylabel('Density',fontsize=18)
